<?php

namespace Controller;

use Service\Traits\TResponse;
use Service\DeleteCascadeService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Utilities;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Notification;

class DefaultColumns extends Crud{
    
    use TResponse;

    protected $dataSourceName = "defaultColumns";

    protected $defaultColumnsService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, \Service\DefaultColumns $defaultColumnsService)
    {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->defaultColumnsService = $defaultColumnsService;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            Utilities::convertDynamicFilterToArray($params);
            Utilities::convertBoolFilterToBoolean($params, ['isDate', 'isFunctionAccumulative']);
            $defaultColumns = Utilities::parseDataSourceZeedhiNext($this->defaultColumnsService->getDefaultColumns($params), $this->defaultColumnsService->count($params));
            $defaultColumns['data'] = $this->fillEmptyAttributes($defaultColumns['data']);
            $defaultColumns['pagination']['page'] = array_key_exists('page', $params) ? intval($params['page']) : 1;
            $defaultColumns['pagination']['limit'] = array_key_exists('limit', $params) ? intval($params['limit']) : 99999;
            $this->sendResponse($defaultColumns, 200);
            //$response->addDataSet(new DataSet('defaultColumns', $defaultColumns));
        } catch (Exception $e) {
        }
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        foreach ($rows as &$row) {
            if (array_key_exists('isDate', $row)) {
                $row['isDate'] = $row['isDate'] == "S";
            }
            if (array_key_exists('isFunctionAccumulative', $row)) {
                $row['isFunctionAccumulative'] = $row['isFunctionAccumulative'] == "S";
            }
        }
        try {
            $dataSet = new DataSet($this->dataSourceName, $rows, $dataSet->getPagination());
            $persistedRows = $this->mongoManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            $response->addNotification(new Notification("{$nrPersistedRows} row(s) persisted with success", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function updateOrInsertDefault(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $persistedRows = $this->defaultColumnsService->updateOrInsertDefaultColumn($row);
            $nrPersistedRows = count($persistedRows);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $response->addNotification(new DTO\Response\Notification("{$nrPersistedRows} columns subscribed with success"));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    private function fillEmptyAttributes($defaultColumns)
    {
        foreach ($defaultColumns as $key => $defaultColumn) {
            $defaultColumns[$key]['type'] = array_key_exists('type', $defaultColumn) ? $defaultColumn['type'] : '';
            $defaultColumns[$key]['isDate'] = (array_key_exists('isDate', $defaultColumn) && $defaultColumn['isDate']) ? "S" : "N";
            $defaultColumns[$key]['isFunctionAccumulative'] = (array_key_exists('isFunctionAccumulative', $defaultColumn) && $defaultColumn['isFunctionAccumulative']) ? "S" : "N";
        }

        return $defaultColumns;
    }

    private function formatServerQuery($serverQuery)
    {
        $paramsString = explode('&', $serverQuery);
        $params = array();
        if ($paramsString && $paramsString[0]) {
            foreach ($paramsString as $key => $value) {
                $parts = explode('=', $value);
                if (strpos($parts[0], '%5B') !== false) {
                    $newParts = explode('%5B', $parts[0]);
                    $parts[0] = $newParts[0];
                }
                if ($parts[1] == "true") $parts[1] = true;
                else if ($parts[1] == "false") $parts[1] = false;
                if (!isset($params[$parts[0]])) {
                    $params[$parts[0]] = $parts[1];
                } else {
                    if (!is_array($params[$parts[0]])) $params[$parts[0]] = array($params[$parts[0]]);
                    array_push($params[$parts[0]], $parts[1]);
                }
            }
        }

        return $params;
    }

    private function comparatorAsc($object1, $object2)
    {
        return strcmp(strtolower($object1[$this->orderField]), strtolower($object2[$this->orderField]));
    }

    private function comparatorDesc($object1, $object2)
    {
        return strcmp(strtolower($object2[$this->orderField]), strtolower($object1[$this->orderField]));
    }
}
