<?php

namespace Controller;

use Service\Traits\TResponse;
use Service\DeleteCascadeService;
use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;

class DefaultRelations extends Crud{
    
    use TResponse;

    protected $dataSourceName = "defaultRelations";

    protected $defaultRelationsService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, \Service\DefaultRelations $defaultRelationsService) {
        $mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->defaultRelationsService = $defaultRelationsService;
    }
    
    public function findNext(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        $filter = new ZmartFilterCriteria($this->dataSourceName, array());
        
        // Filter by field
        if (array_key_exists("in", $params)) {
            foreach ($params["in"] as $paramFilter){
                if(!array_key_exists($paramFilter, $params))
                    continue;
                $operator = FilterCriteria::EQ;
                $filterValue = $params[$paramFilter];
                if (is_string($filterValue) && $paramFilter != "structureType"){
                    $filter->addCondition($paramFilter, FilterCriteria::LIKE_I, $filterValue);
                }
                else if (is_array($filterValue) && count($filterValue) > 0){
                    foreach ($filterValue as $value)
						$filter->addCondition($paramFilter, FilterCriteria::IN, $value);
                }
                else {
                    $filter->addCondition($paramFilter, $filterValue);
                }
            }
        }
        
        $dataSet = $this->mongoManager->findBy($filter);
        $response->addDataSet($dataSet);
        $data = current($response->getDataSets())->getRows();
        
        $total = count($data);
        $limit = isset($params["limit"]) ? (int)$params["limit"] : 9999;
        $page = isset($params["page"]) ? (int)$params["page"] : 1;
        $data = array_chunk($data, $limit);
        if ($page > count($data))
            $page = count($data);
        if ($page <= 0)
            $page = 1;
        if (count($data) > 0)
            $data = $data[$page-1];
            
        $this->sendResponse(["data" => array(
	        "defaultRelations" => $data,
	        "pagination" => ["total" => $total, "page" => $page, "limit" => $limit]
        )], 200);
    }

    public function updateOrInsertDefault(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $persistedRows = $this->defaultRelationsService->updateOrInsertDefaultRelation($row);
            $nrPersistedRows = count($persistedRows);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $response->addNotification(new DTO\Response\Notification("{$nrPersistedRows} relations subscribed with success"));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}