<?php

namespace Controller;

use Service\Traits\TResponse;
use Zeedhi\Framework\DataSource\Manager;
use Service\DeleteCascadeService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Utilities;

class Goals extends Crud {

    use TResponse;

    protected $dataSourceName = 'goals';
    protected $goalsService;
    protected $goalsNotificationsService;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService,
        \Service\Goals $goalsService, \Service\GoalNotifications $goalsNotificationsService)
    {
        $this->goalsService = $goalsService;
        $this->goalsNotificationsService = $goalsNotificationsService;
        parent::__construct($organizationMongoManager, $deleteCascadeService);
    }

    public function findNext(DTO\Request $request, DTO\Response $response){
        try{
            $params = $request->getParameters();
            if (array_key_exists("viewId", $params)) {
                Utilities::convertDynamicFilterToArray($params);
                $params["dynamic_filter"]["viewId"] = [["relation"=>  "AND", "operation" => "EQUALS", "value"=> $params["viewId"]]];
                $data = $this->goalsService->findNext($params, false);
                $data = Utilities::parseDataSourceZeedhiNext($data, $this->goalsService->countNext($params, false));
            } else {
                $data = Utilities::parseDataSourceZeedhiNext([], 0);
            }
            // Paginação
            $data["pagination"]["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
            $data["pagination"]["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
            $this->sendResponse($data, 200);
        }catch (\Exception $e){
            throw  $e;
        }
    }


    public function removeGoals(DTO\Request $request, DTO\Response $response){
        $request = $this->buildDataSetIfRow($request);
        $dataSet = $request->getDataSet()->getRows();
        $goals = $dataSet[0]["goals"];
        $this->removeGoalsAndDependences($goals);
    }

    public function removeGoalsAndDependences($goals){
        foreach($goals as $goal){
            $goalId = $goal["_id"];
            $dependences = $this->goalsNotificationsService->getGoalNotificationByGoalId($goal["_id"]);
            $quantityDeleted = $this->goalsNotificationsService->delete($dependences);

            if(sizeof($dependences) == sizeof($quantityDeleted)){
                $this->goalsService->delete(array($goal));
            }
        }
        
    }

    protected function buildDataSetIfRow(DTO\Request $request) {
		if ($request instanceof DTO\Request\Row) {
			$dataSet = new DataSet($this->dataSourceName, array($request->getRow()));
			$request = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
		}
		return $request;
	}
}