<?php
namespace Controller;

use Service\DeleteCascadeService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class ManualImportQueue extends Crud {

	protected $dataSourceName = 'manualImportQueue';

	/** @var \Service\ManualImportQueueService $queueService */
	protected $queueService;

	public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService,
								$queueService) {
		parent::__construct($mongoManager, $deleteCascadeService);
		$this->queueService = $queueService;
	}

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
		$filterData = $request->getFilterCriteria();
		$filterData->setDataSourceName($this->dataSourceName);
		$filterData->addOrderBy('order', FilterCriteria::ORDER_ASC);
		parent::find($request, $response);
	}

	public function save(DTO\Request $request, DTO\Response $response) {
		$request = $this->buildDataSetIfRow($request);
		$itemsToAddToQueue = array();

		foreach ($request->getDataSet()->getRows() as $row) {
			if(isset($row['scheduleItem'])) {
				$row = $row['scheduleItem'];
				$this->setProcessOrder($row);
				array_push($itemsToAddToQueue, $row);
			} else {
				array_push($itemsToAddToQueue, array());
			}
		}

		$return = $this->queueService->saveQueue($itemsToAddToQueue);
		$response->addDataSet(new DataSet('Queue', $return));
	}

	protected function setProcessOrder(&$row) {
		$maxOrder = $this->queueService->getMaxOrder($row['dataSource']);
		$row['order'] = $maxOrder +1;
	}

	public function delete(DTO\Request $request, DTO\Response $response) {
		$request = $this->buildDataSetIfRow($request);
		$row = $request->getDataSet()->getRows()[0];
		$itemOrder = $row['order'];

		parent::delete($request, $response);
		$this->updateProcessesOrders($itemOrder);
	}

	public function updateProcessesOrders($order) {
		$itemsToUpdate = $this->queueService->findByConditions('order', $order, FilterCriteria::GT);
		$this->queueService->updateProcessesOrders($itemsToUpdate);
	}
}