<?php

namespace Service\ChartView;

class CardDetail extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_CARD_DETAIL = "581b72bde237895b2ec91916";

    public $widget;
    public $widgetData;

    public $textColor;
    public $cardColor;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function fontShouldBeWhite($color) {
        $c = substr($color,1); // strip #
        $rgb = intval($c, 16); // convert rrggbb to decimal
        $r = ($rgb >> 16) & 0xff; // extract red
        $g = ($rgb >> 8) & 0xff; // extract green
        $b = ($rgb >> 0) & 0xff; // extract blue

        $luma = 0.2126 * $r + 0.7152 * $g + 0.0722 * $b;
        return $luma < 155;
    }

    protected function getCardTitle(){
        $widget = $this->widget;
        $customCss = !empty($widget["CSS_TITLE"]) ? $widget["CSS_TITLE"] : "";
        $cardTitle = [
            "name" => "cardTitle",
            "component" => "TekCardTitle",
            "title" => !empty($widget["cardTitle"]) ? $widget["cardTitle"] : $widget["label"],
            "cssClass" => "zmartCardTitle_".$this->textColor,
            "events" => [
                "onMounted" => "{{ChartViewController.cardTitleOnMounted}}"
            ]
        ];
        if(!empty($widget["showPeriod"]) && $widget["showPeriod"] == "Y"){
            $cardTitle["subtitle"] = "De ".$widget["period"]["start"]." a ".$widget["period"]["end"];
        }
        if(!empty($customCss)){
            $cardTitle["cssStyle"] = $customCss;
        }

        return $cardTitle;
    }

    protected function getCardValue(){
        $widget = $this->widget;
        $widgetData = $this->widgetData;
        $cardValue = !empty($widgetData[0][$widget["cardValue"]]) ? $widgetData[0][$widget["cardValue"]] : 0;
        $customCss = !empty($widget["CSS_VALUE"]) ? $widget["CSS_VALUE"] : "";
        return [
            "name" => "cardValue",
            "component" => "ZdText",
            "cssClass" => "zd-text-center zmartCard",
            "cssStyle" => "color: ".$this->textColor.";height: 70%;$customCss",
            "text" => "<div class='zd-display-flex zd-justify-center' style='height:100%;'><div class='zd-align-self-center' style='font-size:56px;'>".strval($cardValue)."</div></div>",
        ];
    }

    protected function getBottomRow(){
        $cardValue2 = !empty($this->widgetData[0][$this->widget["cardValue2"]]) ? $this->widgetData[0][$this->widget["cardValue2"]] : 0;
        $cardValue3 = !empty($this->widgetData[0][$this->widget["cardValue3"]]) ? $this->widgetData[0][$this->widget["cardValue3"]] : 0;
        $cardTitle2 = !empty($this->widget["cardTitle2"]) ? $this->widget["cardTitle2"] : "";
        $cardTitle3 = !empty($this->widget["cardTitle3"]) ? $this->widget["cardTitle3"] : "";
        return [
            "name" => "cardRow",
            "component" => "ZdRow",
            "justify" => "space-around",
            "children" => [
                $this->getBottomCard($cardTitle2, $cardValue2),
                $this->getBottomCard($cardTitle3, $cardValue3),
            ],
        ];
    }

    protected function getBottomCard($title, $value){
        return [
           "name" => "bottomCard",
           "component" => "ZdCard",
           "flat" => true,
           "color" => $this->cardColor,
           "cssStyle" => "width: 45%;",
           "children" => [
                [
                    "name" => "cardTitle",
                    "component" => "TekCardTitle",
                    "title" => $title,
                    "cssClass" => "zmartCardTitle_".$this->textColor,
                    "events" => [
                        "onMounted" => "{{ChartViewController.cardTitleOnMounted}}"
                    ]
                ],
                [
                    "name" => "cardValue",
                    "component" => "ZdText",
                    "cssClass" => "zd-text-center zmartCard",
                    "cssStyle" => "color: ".$this->textColor.";height: 10%;",
                    "text" => "<div class='zd-display-flex zd-justify-center' style='height:100%;'><div class='zd-align-self-center' style='font-size:28px;'>".strval($value)."</div></div>",
                ],
           ]
        ];
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;
        $widgetData = $this->widgetData;

        $cardColor = !empty($widget["cardColors"]) ? $widget["cardColors"] : "";
        if(is_array($cardColor)){
            $cardColor = $cardColor[0];
        }
        $this->textColor = $this->fontShouldBeWhite($cardColor) ? "white" : "black";
        $this->cardColor = $cardColor;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_CARD_DETAIL,
            "component" => self::CARD_COMPONENT,
            "isCardDetail" => true,
            "color" => $this->cardColor,
            "height" =>"100%",
            "cssStyle" => self::CHART_CSS_STYLE,
            "children" => [
                $this->getCardTitle(),
                $this->getCardValue(),
                $this->getBottomRow()
            ],            
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
