<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Gauge extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_GAUGE = "57ab6a47574698ae9522f97e";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
        $this->sortedRangeValues = [];
    }

    protected function getSeries(){
        $widget = $this->widget;
        $widgetData = $this->widgetData;
        return  [
            [
              'type' => 'gauge',
              'radius' => "160%",
              "center" => ["50%","90%"],
              'startAngle' => 180,
              'endAngle' => 0,
              "max" => end($this->sortedRangeValues),
              "min" => reset($this->sortedRangeValues),
              'axisLine' => [
                'lineStyle' => [
                  'width' => 30,
                  "color" => $this->getColors(),
                ],
              ],
              'axisTick' => [
                'show' => false,
              ],
              'pointer' => [
                'itemStyle' => [
                  'color' => 'auto',
                ],
              ],
              'axisLabel' => [
                'color' => 'auto',
                'distance' => 40,
                'fontSize' => 20,
              ],
              'detail' => [
                'valueAnimation' => true,
                'formatter' => '{value}',
                'color' => 'auto',
                "offsetCenter" => [0, "10%"],
              ],
              'data' => [
                [
                  'value' => $this->widgetData[0][$widget["serieValue"]],
                ],
              ],
            ],
        ];
    }

    protected function sortRangeValues(){
        $orderedValues = [];
        foreach($this->widget["zmartSerieRanges"] as $value){
            $orderedValues[$value] =  $this->widgetData[0][$value];
        }
        asort($orderedValues);
        $this->sortedRangeValues = $orderedValues;
    }

    protected function getColors(){
        $widget = $this->widget;
        $widgetData = $this->widgetData;
        $maxValue = end($this->sortedRangeValues);
        $minValue = reset($this->sortedRangeValues);
        $serieColors = $widget["serieColors"];
        $colors = [];
        $i = 0;
        foreach($this->sortedRangeValues as $value){
            if($value != $minValue){
                $colors[] = [$value/$maxValue, $serieColors[$i % count($serieColors)]];
                $i++;
            }
        }
        return $colors;
        
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        $showDataLabel = true;
        $showAsPercent = false;
        if(!empty($widget["serieShowValueLabels"])){
            $showDataLabel = $widget["serieShowValueLabels"] != "false";
            $showAsPercent = $widget["serieShowValueLabels"] == "%";
        }
        return [
            "enabled" => $showDataLabel,
            "position" => "top",
            "textAnchor" => "middle",
            "showAsPercent" => $showAsPercent,
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;

        return [
            "text" => $widget['name'],
            "textStyle" => [
                "fontSize" => "16px"
            ]
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
              "fontSize" => '14px',
            ],
        ];
    }

    protected function getOptions(){
        $this->sortRangeValues();
        return [
            "title" => $this->getTitle(),
            'series' => $this->getSeries(),
        ];
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;
        $widget["gaugeData"] = $this->widgetData[0];

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_GAUGE,
            "component" => "ZdECharts",
            "height" =>"100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
