<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Pie extends ChartViewAbstract implements ChartViewInterface
{

    const TYPE_ID_PIE = "579106aa11e8f41bb00caff7";

    public $widget;
    public $widgetData;

    function __construct($widget = null)
    {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getXAxis()
    {
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            $categoria[] = $row[$widget['chartXAxisField']];
        }
        $xAxis = [
            "mask" => !empty($widget['chartXAxisFormat']) ? $widget['chartXAxisFormat'] : null,
            "categories" => $categoria,
            "field" => $widget['chartXAxisField'],
            "labels" => [
                "show" => true,
                "rotate" => !empty($widget['chartXAxisTicksRotation']) ? $widget['chartXAxisTicksRotation'] - 90 : -60,
            ],
            "tickPlacement" => "between"
        ];

        if ($xAxis["labels"]["rotate"] != 0) {
            $xAxis["labels"]["rotateAlways"] = false;
        }

        return $xAxis;
    }

    protected function getYAxis()
    {
        $widget = $this->widget;
        $serieYAxisFormat = !empty($widget['chartSeries'][0]['serieValueFormat']) ? $widget['chartSeries'][0]['serieValueFormat'] : null;
        $yAxis = [
            "mask" => $serieYAxisFormat,
            "dataLabels" => [],
            "labels" => [
                "show" => true,
            ],
            "opposite" => !empty($widget["serieYAxisPosition"]) ? $widget["serieYAxisPosition"] == "right" : false,
        ];

        if (!empty($widget['serieYAxisLabel'])) {
            $yAxis['title'] = [
                "text" => $widget['serieYAxisLabel'],
                "style" => [
                    "fontSize" => "16px"
                ]
            ];
        }

        return $yAxis;
    }

    /* protected function getSeries(){
        $widget = $this->widget;
        $series = [];
        foreach ($this->widgetData as $row) {
            $fact = $widget["serieFieldValue"];
            $series[] = !empty($row[$fact]) ? $row[$fact] : 0;
        }
        return $series;
    } */

    protected function getSeries()
    {
        $widget = $this->widget;
        $widgetDatas = $this->widgetData;
        return  [
            [
                'name' => 'Access From',
                'type' => 'pie',
                'radius' => "50%",
                "center" => ["50%", "50%"],
                'data' => $this->getDataPie(),
                'emphasis' => [
                    'itemStyle' => [
                        'shadowBlur' => 10,
                        'shadowOffsetX' => 0,
                        'shadowColor' => 'rgba(0, 0, 0, 0.5)'
                    ]
                ]
            ],
        ];
    }

    protected function getDataPie()
    {
        $widget = $this->widget;
        $widgetDatas = $this->widgetData;
        $arrayAux = [];
        foreach ($widgetDatas as $widgetData) {
            $value = $widgetData[$widget["serieFieldValue"]];
            $name = $widgetData[$widget["chartXAxisField"]];
            if ($widget["serieShowValueLabels"] == "%") {
                $valor_base = 4000;
                $valor = $widgetData[$widget["serieFieldValue"]];
                $resultado = ($valor / $valor_base) * 100;
                $value = $resultado;
            }
            $arrayAux[] =
                [
                    'name'  => $name,
                    'value' => $value
                ];
        }
        if ($widget["serieSort"] == 'label') {
            $arrayAux = $this->ordenarArray($arrayAux, 'name');
        } else if ($widget["serieSort"] == 'value') {
            $arrayAux = $this->ordenarArray($arrayAux, 'value');
        }
        return $arrayAux;
    }

    function ordenarArray($array, $campo) {
        $comparar = function ($a, $b) use ($campo) {
            if ($campo === 'name') {
                return strcmp($a['name'], $b['name']);
            } elseif ($campo === 'value') {
                if ($a['value'] < $b['value']) {
                    return -1;
                } elseif ($a['value'] > $b['value']) {
                    return 1;
                } else {
                    return 0;
                }
            }
            return 0;
        };
        usort($array, $comparar);
        return $array;
    }
    
    protected function getLabels()
    {
        $widget = $this->widget;
        $labels = [];
        foreach ($this->widgetData as $row) {
            $dimension = $widget["serieFieldLabel"];
            $labels[] = !empty($row[$dimension]) ? $row[$dimension] : "";
        }
        return $labels;
    }

    protected function getTooltip()
    {
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions()
    {
        $widget = $this->widget;
        $pieOptions = [
            "pie" => [
                "labels" => [
                    "show" => true,
                    "name" => [
                        "show" => true,
                    ],
                    "value" => [
                        "show" => true,
                    ],
                ]
            ]
        ];

        return [
            "pie" => $pieOptions
        ];
    }

    protected function getSerieStacked()
    {
        $widget = $this->widget;

        if (!empty($widget['serieStacked'])) {
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart()
    {
        $widget = $this->widget;

        return [
            "type" => "pie",
            "id" => $widget['id'],
            "stacked" => $this->getSerieStacked(),
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors()
    {
        $widget = $this->widget;
        if (!empty($widget["chartSeries"][0]["serieColors"])) {
            return $widget["chartSeries"][0]["serieColors"];
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels()
    {
        $widget = $this->widget;
        $showDataLabel = false;
        $showAsPercent = false;
        if (!empty($widget["serieShowValueLabels"])) {
            $showDataLabel = $widget["serieShowValueLabels"] != "false";
            $showAsPercent = $widget["serieShowValueLabels"] == "%";
        }
        return [
            "enabled" => $showDataLabel,
            "position" => "top",
            "textAnchor" => "middle",
            "showAsPercent" => $showAsPercent,
        ];
    }

    protected function getTitle()
    {
        $widget = $this->widget;
        $string = $widget["chartLegendPosition"];
        $search = "bottom";
        $top = "";
        if (stripos($string, $search) !== false) {
            $top = "bottom";
        }
        return [
            "text" => $widget['name'],
            "style" => [
                "fontSize" => "16px"
            ],
            "left" => "center",
            "top" => $top
        ];
    }

    protected function getNoData()
    {
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
                "fontSize" => '14px',
            ],
        ];
    }

    /* protected function getOptions(){
        $widget = $this->widget;
        $legendPosition = "top";
        if($widget["chartLegendPosition"]){
            $legendPosition = explode(" ",$widget["chartLegendPosition"])[0];
        }

        return [
            "colors" => $this->getSerieColors(),
            "drillDownView" => $widget['drillDownView'],
            "plotOptions" => $this->getPlotOptions(),
            "dataLabels" => $this->getDataLabels(),
            "chart" => $this->getChart(),
            "labels" => $this->getLabels(),
            "legend" => [
                "show" => isset($widget["chartLegendClosed"]) ? $widget["chartLegendClosed"] !== "removeLegend" : true,
                "position" => $legendPosition
            ],
            "zoom" > [
                "enabled" => true
            ],
            "title" => $this->getTitle(),
            "noData" => $this->getNoData(),
            "tooltip" => $this->getTooltip(),
            "xaxis" => $this->getXAxis(),
            "yaxis" => $this->getYAxis()
        ];
    } */

    protected function getOptions()
    {
        return [
            "legend" => $this->getLegend(),
            "title" => $this->getTitle(),
            "tooltip" => [
                "trigger" => "item",
            ],
            'series' => $this->getSeries(),
        ];
    }

    protected function getLegend()
    {
        $widget = $this->widget;
        $string = $widget["chartLegendPosition"];
        $variable = [
            'top' => "",
            'inf' => "",
            'left' => "",
            'right' => ""
        ];
        $variations = explode(" ", $string);
        foreach ($variations as $variation) {
            switch ($variation) {
                case "top":
                    $variable['top'] = "bottom";
                    break;
                case "bottom":
                    $variable['inf'] = "bottom";
                    break;
                case "left":
                    $variable['left'] = "center";
                    break;
                case "right":
                    $variable['right'] = "center";
                    break;
            }
        }
        return $variable;
    }

    public function convertToNext(): array
    {
        $this->fetchWidgetData();
        $widget = $this->widget;
        $widget["pieData"] = $this->widgetData[0];
        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_PIE,
            "component" => "ZdECharts",
            "height" => "100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
