<?php
namespace Service;

use Util\Environment;
use Zeedhi\Notification\API as NotificationAPI;
use Service\Mail\IndicatorMail;
use Service\Mail\DataSourceMail;
use Service\Mail\GoalsMail;

class Notification {

    const EMAIL_OF_INDICATOR = "indicator";
    const EMAIL_OF_DATASOURCE = "dataSource";
    const EMAIL_OF_GOALS_NOTIFICATIONS = "goalNotifications";

    protected $environment;
    protected $notificationApi;
    protected $indicatorMail;
    protected $dataSourceMail;
    protected $goalsMail;

    public function __construct(Environment $environment, NotificationAPI $notificationApi,
                                IndicatorMail $indicatorMail, DataSourceMail $dataSourceMail,
                                GoalsMail $goalsMail) {
        $this->environment = $environment;
        $this->notificationApi = $notificationApi;
        $this->indicatorMail = $indicatorMail;
        $this->dataSourceMail = $dataSourceMail;
        $this->goalsMail = $goalsMail;
    }

    public function sendNotification($title, $message, $indicatorPath) {
        $organizationId = $this->environment->getOrganizationId();

        $notification = array(
            'title' => $title,
            'message' => $message,
            'data'  => array('indicatorPath' => $indicatorPath)
        );
        $this->notificationApi->sendMessage($organizationId, $notification);
    }

    public function sendMailNotification($data, $type){
        switch ($type) {
            case self::EMAIL_OF_DATASOURCE:
                $to = $data["mail"];
                $name = $data["name"];
                $message = $data["message"];
                $dataSourceName = $data["dataSourceName"];

                $this->dataSourceMail->sendMessageMail($to, $name, $message, $dataSourceName);
                break;
            case self::EMAIL_OF_INDICATOR:
                $to             = $data["to"];
                $sender         = $data["sender"];
                $message        = $data["message"];
                $date           = $data["sendDate"];
                $nameIndicator  = $data["nameIndicator"];
                $indicator      = $data["indicator"];
                $messageType    = $data["messageType"];

                $this->indicatorMail->sendMessageMail($to, $sender, $message, $date, $nameIndicator,
                    $indicator, $messageType);
                break;
            case self::EMAIL_OF_GOALS_NOTIFICATIONS:
                $to               = $data["to"];
                $labelView        = $data["labelView"];
                $dataSource       = $data["dataSource"];
                $lastUpdate       = $data["lastUpdate"];
                $labelGoal        = $data["labelGoal"];
                $goal             = $data["goal"];
                $value            = $data["value"];
                $userResponsible  = $data["userResponsible"];
                $notificationText = $data["notificationText"];
                $notifyCondition  = $data["notifyCondition"];

                $this->goalsMail->sendMessageMail($to, $labelView, $dataSource, $lastUpdate,
                    $labelGoal, $goal, $value, $userResponsible,
                    $notificationText, $notifyCondition);
                break;
            default:
                break;
        }
    }


}